/****************************************************************************
 **                                                                        **
 **    File:           cmds.c                                              **
 **    Author:         Roger Preston                                       **
 **    Created/Copied: Wed Dec 14 17:56:00 EST 1988                        **
 **    Contents :      Subset of commands for GP1000 stress test           **
 **                                                                        **
 **    Copyright (c) 1988 BBN Communications Corp.                         **
 **    All rights reserved.                                                **
 **                                                                        **
 ****************************************************************************/
#include <stdio.h>
#include <signal.h>
#include "tmp4.h"
#include "tmp3.h"
#include "tmp1.h"

extern FILE *do_file;
extern struct dump_cmd_definition dcmd_def[];
extern int search_dcmd();

struct child_def
{
  char *cname;
  char *carg[7];
};

struct child_def cd[] = {
  "disk_io", {NULL, NULL, NULL, NULL, NULL, NULL, NULL},
  "scsi_io", {NULL, NULL, NULL, NULL, NULL, NULL, NULL},
  "ethr_io", {NULL, NULL, NULL, NULL, NULL, NULL, NULL},
  "tape_io", {NULL, NULL, NULL, NULL, NULL, NULL, NULL},
  "term_io", {NULL, NULL, NULL, NULL, NULL, NULL, NULL},
  NULL,      {NULL, NULL, NULL, NULL, NULL, NULL, NULL}
};

struct cpid pid_array[MAXNUMCHLD];

char mx_str[] = "matrix";
 
void syserr();
void add_pid();



/*****************************************************************************
 * "Do" command - Set up to run a command script file.                       *
 *****************************************************************************/
dcm_do(ilp)
register struct input_line *ilp;
{
register unsigned long val;
int res;

  if (ilp->argc != 2) 
  {
    usage(ilp);
    return(FALSE);
  }

  if (do_file != (FILE *)NULL) 
  {
    printf(" Truncating previous script file.\n");
    fclose(do_file);
    do_file = (FILE *)NULL;
  }

  do_file = fopen(ilp->arg[1], "rb");     /* "b" = binary, must handle */
                                          /* control-Z and CR/LF      */
  if (do_file == (FILE *)NULL) 
  {
    printf(" Can't open command file '%s'.\n", ilp->arg[1]);
    return(FALSE);
  } 
  else 
  {
    printf(" Executing commands from '%s'.\n", ilp->arg[1]);
    return(TRUE);
  }
}




/*****************************************************************************
 * Exit from console program                                                 *
 *****************************************************************************/
dcm_exit(ilp)
struct input_line *ilp;
{
  printf("Exit to MACH.\n");
  exit(0);
}


/*****************************************************************************
 * Unimplemented command stub                                                *
 *****************************************************************************/
dcm_notyet(ilp)
struct input_line *ilp;
{
  printf("This command is not yet implemented.\n");
  usage(ilp);
  return(FALSE);
}


/*****************************************************************************
 * dcm_help() -- if no arguments, print a list of commands that the dumper   *
 *   will accept; if 1 argument, print help message for that argument        *
 *****************************************************************************/
dcm_help(ilp)
struct input_line *ilp;
{
  short i;
  short per_line;
  short index;
  
extern char *long_help[];

  if (ilp->argc > 2)
  {
    usage(ilp);
    return(FALSE);
  }

  switch(ilp->argc)
  {
    case 1:
      printf("commands:\n ");
      per_line = 0;
      for(i = 0; dcmd_def[i].cmd_name; i++) 
      {
        printf("%s  ", dcmd_def[i].cmd_name);
        if (++per_line == 6)
        {
          per_line = 0;
          printf("\n ");
        }
      }
      printf("\n");
      break;

    case 2:
      i = 1;
      while (i < ilp->argc)
      {
        if ((index = search_dcmd(ilp->arg[i])) != UNDEFINED)
        {
          if (dcmd_def[index].help_msg != NULL)
	  {
            printf("%s: %s\n", dcmd_def[index].cmd_name, 
                               dcmd_def[index].help_msg);
            printf("%s", long_help[index]);
	  }
          else
            printf("%s: No help available\n", dcmd_def[index].cmd_name);
        }
        else
          printf("%s: Unknown command; no help...\n", ilp->arg[i]);
        i++;
      }
      break;

    default:
      usage(ilp);
      break;
  }
}



/*****************************************************************************
 * dcm_fork - fork an I/O intensive program for one of the 5 peripheral      *
 *   subsystems...                                                           *
 *****************************************************************************/
dcm_fork(ilp)
register struct input_line *ilp;
{
  int pid, child_index, match;
  int i,j;
  
  if (ilp->argc > 4)    /* Max. # of args 'fork' can have is 4 */
  {
    usage(ilp);         /* usage statement */
    return(FALSE);
  }

/*
 * Search the command-line argument list for one of 'disk_io', 'scsi_io',
 * etc. (cd[] struct)  If not found, print usage statement.  Commands
 * can be of the forms:
 *        fork xxxx_io
 *        fork -r xxxx_io
 *        fork -f xxxx_io
 *        fork -r -f xxxx_io
 *        fork -rf [-fr] xxxx_io
 */

  child_index = -1;
  match = 0;
  for (i = 1; i < ilp->argc; i++)  /* for each command line argument */
  {
    for (j = 0; cd[j].cname != NULL; j++)    
    {                                   
      if (smatch(ilp->arg[i], cd[j].cname) == EXACT_MATCH) /* SUBSTRING...*/
      {
        child_index = j;
        match++;
        break;
      }
    }
  }

  if (match != 1)
  {
    printf("Unknown argument combination...\n");
    usage(ilp);
    return(FALSE);
  }

/*
 * Make a moderate attempt to weed out illegal combinations or duplicate
 * arguments (i.e., 1!=2,1!=3,1!=4,1!=5,2!=3,2!=4...4!=5).  Will disallow
 * commands of the form: "fork disk_io disk_io" which should instead
 * be entered via: "fork disk_io" followed by "fork disk_io".
 */
  for (i = 1; i < ilp->argc-1; i++)
  {
    for (j = i+1; j < ilp->argc; j++)
    {
      if (smatch(ilp->arg[i], ilp->arg[j]) == EXACT_MATCH)
      {
        printf("Illegal combination or duplicate argument...\n");
        usage(ilp);
        return(FALSE);
      }
    }
  }

  if (child_index >= 0)  /* 'xxxx_io' pattern found... */
  {
    ilp->arg[0] = cd[child_index].cname;

    switch (pid = fork())   /* fork the child; child will 'exec' the I/O */
    {
      case ERROR: /* parent -- failure */
        printf("Can't create child process\n");
        return(FALSE);

      case 0:   /* child */
        execvp(cd[child_index].cname, ilp->arg);
        printf("Child was unable to 'exec' new program (same process)\n");
        exit(1);

      default:  /* parent -- success */
/*      printf("pid of child = %d\n", pid);  */    /* Do NOT wait for child */
        add_pid(pid, "fork");
        return(TRUE);                   /* Later: implement -w wait flag */
    }
  }
  else
  {
    usage(ilp);
    return(FALSE);
  }
}




/*****************************************************************************
 * dcm_combine - combine I/O intensive activity among two or three I/O       *
 *   subsystems                                                              *
 *****************************************************************************/
dcm_combine(ilp)
register struct input_line *ilp;
{
  int i,j;
  int pid, match;

/* 
 * Check for correct number of arguments...
 */
  if (ilp->argc > 6)
  {
    usage(ilp);
    return(FALSE);
  }

/*
 * Do some argument checking: first check for presence of at
 * least two and no more than three DISTINCT xxxx_io's.  Commands
 * can be of the form:
 *        combine xxxx_io xxxx_io [xxxx_io] [xxxx_io]
 *        combine -r xxxx_io xxxx_io [xxxx_io] [xxxx_io]
 *        combine -f xxxx_io xxxx_io [xxxx_io] [xxxx_io]
 *        combine -rf [-fr] xxxx_io xxxx_io [xxxx_io] [xxxx_io]
 */
  match = 0;
  for (i = 1; i <= 5; i++)   /* for each command line argument ... */
  {
    for (j = 0; cd[j].cname != NULL; j++)
    {
      if (smatch(ilp->arg[i], cd[j].cname) == EXACT_MATCH)
      {
        match++;
        break;
      }
    }    
  }

  if (match < 2 || match > 4)
  {
    printf("Illegal combination...\n");
    usage(ilp);
    return(FALSE);
  }


/*
 * Next, check that no arguments are duplicated...
 * (someday, check for SUBSTRING match too)
 */
  for (i = 1; i < ilp->argc-1; i++)
  {
    for (j = i+1; j < ilp->argc; j++)
    {
      if (smatch(ilp->arg[i], ilp->arg[j]) == EXACT_MATCH)
      {
        printf("Illegal combination or duplicate argument...\n");
        usage(ilp);
        return(FALSE);
      }
    }
  }

  switch(pid = fork())
  {
    case ERROR:
      printf("Can't create child process\n");
      return(FALSE);

    case 0:
      execvp("combine", ilp->arg);
      printf("Child was unable to 'exec' new program (same process)\n");
      exit(1);

    default:
/*    printf("pid of child = %d\n", pid); */
      add_pid(pid, "combine");
      return(TRUE);  /* do NOT wait for child */
  }
}



/****************************************************************************
 * dcm_exec --                                                              *
 ****************************************************************************/
dcm_exec(ilp)
register struct input_line *ilp;
{
  int i,status;

  if (ilp->argc < 2)
  {
    usage(ilp);
    return(FALSE);
  }

  for (i = 0; i < N_ARGS; i++)    /* Shift the arguments */
    ilp->arg[i] = ilp->arg[i+1];
  
  ilp->arg[N_ARGS] = NULL;        /* Make the last argument a zombie */

  for (i = 0; i < N_ARGS; i++)
    printf("ilp->arg[%02d] = >%s<\n", i, ilp->arg[i]);    

  switch(fork())
  {
    case ERROR:
      printf("Can't create child process\n");
      return(FALSE);

    case 0:
      execvp(ilp->arg[0], ilp->arg);  /* SOMEDAY, UPDATE ENV! (see Rochkind) */
      printf("Child was unable to 'exec' new program (same process)\n");
      exit(1);

    default:
      if (wait(&status) == ERROR) /* SOMEDAY: -- implement a better wait; */
        syserr("wait", TRUE);     /* implement flag (-w) to wait or not */
      return(TRUE);
  }
}


/*
 * print system call error message and (possibly) terminate
 */
void syserr(msg, t)
char *msg;
int t;
{
  extern int errno, sys_nerr;
  extern char *sys_errlist[];

  fprintf(stderr, "ERROR: %s (%d",  msg, errno);
  if (errno > 0 && errno < sys_nerr)
    fprintf(stderr, "; %s)\n", sys_errlist[errno]);
  else
    fprintf(stderr, ")\n");
  if (t)
    exit(1);
}



int dcm_ioctl(ilp)
register struct input_line *ilp;
{
  return(FALSE);
}
int dcm_syscall(ilp)
register struct input_line *ilp;
{
  return(FALSE);
}

/***************************************************************************
 * dcm_unisys --                                                           *
 *        Do Uniform System Matrix Multiply or Image Convolution           *
 ***************************************************************************/
int dcm_unisys(ilp)
register struct input_line *ilp;
{
  register int i,j;
  int pid;
  int match = 0;

  if ((ilp->argc > 3) || (ilp->argc < 2))
  {
    usage(ilp);
    return(FALSE);
  }

/*
 * Search the command-line argument list for one of 'mm' or 'ic' (for
 * [m]atrix [m]ultiply & [i]mage [c]onvolution, respectively).
 * If not found, print usage statement.  Commands can be of the forms:
 *        uni_sys mm
 *        uni_sys ic
 *        uni_sys -f mm    (-f flag: loop forever)
 *        uni_sys -f ic
 */

  for (i = 1; i < ilp->argc; i++)  /* for each command line argument */
  {
    if ((smatch(ilp->arg[i], "mm") == EXACT_MATCH) ||      /* SUBSTRING */
              (smatch(ilp->arg[i], "ic") == EXACT_MATCH))
      match++;
  }

  if (match != 1)
  {
    printf("Unknown argument combination...\n");
    usage(ilp);
    return(FALSE);
  }

/*
 * Make a moderate attempt to weed out illegal combinations or duplicate
 * arguments (i.e., 1!=2, 2!=3).  Will disallow commands of the form:
 * "uni_sys mm mm" or "uni_sys ic ic".
 */
  for (i = 1; i < ilp->argc-1; i++)
  {
    for (j = i+1; j < ilp->argc; j++)
    {
      if (smatch(ilp->arg[i], ilp->arg[j]) == EXACT_MATCH)
      {
        printf("Illegal combination or duplicate argument...\n");
        usage(ilp);
        return(FALSE);
      }
    }
  }

  ilp->arg[0] = mx_str;
  switch (pid = fork())
  {
    case ERROR:
      printf("Can't create child process.\n");
      return(FALSE);

    case 0:
      execvp("matrix", ilp->arg);
      printf("Child was unable to 'exec' new program (same process)\n");
      exit(1);

    default:
/*    printf("pid of child = %d\n", pid);  */
      add_pid(pid, "uni_sys");
      return(TRUE);
  }
}


int dcm_iofail(ilp)
register struct input_line *ilp;
{
  return(FALSE);
}
int dcm_slct(ilp)
register struct input_line *ilp;
{
  return(FALSE);
}

int dcm_compile(ilp)
register struct input_line *ilp;
{
  int pid;

  if (ilp->argc > 2)
  {
    usage(ilp);
    return(FALSE);
  }

  switch (pid = fork())
  {
    case ERROR:
      printf("Can't create child process\n");
      return(FALSE);

    case 0:
      execvp("cc_io", ilp->arg);
      printf("Child was unable to 'exec' new program (same process)\n");
      exit(1);

    default:
/*    printf("pid of child = %d\n", pid); */
      return(TRUE);
  }
}

int dcm_nroff(ilp)
register struct input_line *ilp;
{
  int pid;

  if (ilp->argc > 2)
  {
    usage(ilp);
    return(FALSE);
  }

  switch (pid = fork())
  {
    case ERROR:
      printf("Can't create child process\n");
      return(FALSE);

    case 0:
      execvp("nr_io", ilp->arg);
      printf("Child was unable to 'exec' new program (same process)\n");
      exit(1);

    default:
/*    printf("pid of child = %d\n", pid); */
      return(TRUE);
  }
}


/****************************************************************************
 * dcm_kill --                                                              *
 *    kills processes generated by the Stress Test (usually with the -f     *
 *    flag)                                                                 *
 ****************************************************************************/
int dcm_kill(ilp)
register struct input_line *ilp;
{
  register int i,j;
  int x, kill_stat;
  int pgrp, match;

  if (ilp->argc > 2)
  {
    usage(ilp);
    return(FALSE);
  }

/*
 * Allowable forms:
 *   kill ? -- lists current outstanding pid's of children (but not
 *             necessarily grandchildren)
 *   kill pid -- kill specific pid; checked against table of allowable
 *               pid's (user cannot run amok);
 *   kill all -- kill all pid's in table
 *
 * Unlike other dcm_ commands, this one blocks (i.e., it is *not*
 * non-blocking...)
 */

  if (smatch(ilp->arg[1], "?") == EXACT_MATCH)
  {
    printf("Known processes:\n");
    for (i = 0; i < MAXNUMCHLD; i++)
    {
      if (pid_array[i].pid != 0)
        printf("  Name = >%s<   Pid = %d\n", pid_array[i].name,
                      pid_array[i].pid);
    }
  }
  else if (smatch(ilp->arg[1], "all") == EXACT_MATCH)
  {
    if ((kill_stat = kill(0, SIGKILL)) == -1)
      syserr("kill", FALSE);
    printf("Kill status = 0x%x (%d)\n", kill_stat, kill_stat);

    pgrp = getpgrp(0);
    if ((kill_stat = killpg(pgrp, SIGKILL)) == -1)
      syserr("killpg", FALSE);
    printf("Kill status = 0x%x (%d)\n", kill_stat, kill_stat);

    for (i = 0; i < MAXNUMCHLD; i++)
    {
      pid_array[i].pid = 0;
      for (j = 0; j < MAXNAMLEN; j++)
        pid_array[j].name[j] = '\0';
    }
  }

/* else assume arg1 = pid */

  else
  {
    match = 0;
    x = atoi(ilp->arg[1]);
    for (i = 0; i < MAXNUMCHLD; i++)
      if (x == pid_array[i].pid) match++;

    if (match)
    {
      if ((kill_stat = kill(x, SIGKILL)) == -1)
        syserr("kill", FALSE);
      printf("Kill status = 0x%x (%d)\n", kill_stat, kill_stat);
    }
    else
      printf("Unknown pid\n");
  }
}



/*
 *
 */
void add_pid(pd, nm)
int pd;
char *nm;
{
  register int i;
  int add = 0;

  for (i = 0; i < MAXNUMCHLD; i++)
  {
    if (pid_array[i].pid == 0)
    {
      add++;
      pid_array[i].pid = pd;
      scopy(nm, pid_array[i].name);
      break;
    }
  }

  if (!add)
  {
    printf("Fatal error: out of room in process table\n");
    exit(1);
  }
}



/*************************************************************************
 * dcm_sst --                                                            *
 *************************************************************************/
dcm_sst(ilp)
register struct input_line *ilp;
{

  if (ilp->argc > 1)
  {
    usage(ilp);
    return(FALSE);
  }

  scopy("fork", ilp->arg[0]);
  scopy("-f", ilp->arg[1]);

  scopy("disk_io", ilp->arg[2]);
  dcm_fork(ilp);

  scopy("scsi_io", ilp->arg[2]);
  dcm_fork(ilp);

  scopy("ethr_io", ilp->arg[2]);
  dcm_fork(ilp);

  scopy("term_io", ilp->arg[2]);
  dcm_fork(ilp);

  scopy("tape_io", ilp->arg[2]);
  dcm_fork(ilp);
}



