/****************************************************************************
 **                                                                        **
 **    File:           cmdtab.c                                            **
 **    Author:         Roger Preston                                       **
 **    Created:        Wed Dec 14 17:59:48 EST 1988                        **
 **    Contents :      Table of valid commands for stress test             **
 **                                                                        **
 **    Copyright (c) 1988 BBN Communications Corp.                         **
 **    All rights reserved.                                                **
 **                                                                        **
 **                                                                        **
 ****************************************************************************/
#include "tmp1.h"
#include "tmp3.h"


/*****************************************************************************
 *                                                                           *
 * Console Command table - this table defines the commands that will be      *
 * accepted by the console.                                                  *
 *                                                                           *
 *****************************************************************************/

extern int dcm_compile();
extern int dcm_combine();
extern int dcm_do();
extern int dcm_exec();
extern int dcm_exit();
extern int dcm_fork();
extern int dcm_help();
extern int dcm_ioctl();
extern int dcm_iofail();
extern int dcm_kill();
extern int dcm_nroff();
extern int dcm_slct();
extern int dcm_sst();
extern int dcm_syscall();
extern int dcm_unisys();
extern int dcm_quit();

struct dump_cmd_definition dcmd_def[] =
{
    "cc",       dcm_compile, 0x0f,   "[-f] [program]",
    "combine",  dcm_combine, 0x0f,   "[-rf] ????_io ????_io [????_io]",
    "do",       dcm_do,        -1,   "<script_filename> (stress script)",
    "exec",     dcm_exec,      -1,   "<program_/script_ file_name> (MACH)",
    "exit",     dcm_exit,      -1,   "exit to shell level",
    "fork",     dcm_fork,    0x0f,   "[-rfa] ????_io",
    "help",     dcm_help,      -1,   "[command]",
    "ioctl",    dcm_ioctl,     -1,   "?",
    "io_fail",  dcm_iofail,    -1,   "?",
    "kill",     dcm_kill,      -1,   "[?], [pid] [all]",
    "nroff",    dcm_nroff,   0x0f,   "[-f] [program]"
    "quit",     dcm_exit,      -1,   "exit to shell level",
    "select",   dcm_slct,      -1,   "[GP1000] [BFII]",
    "sst",      dcm_sst,       -1,   "start standard ( default) stress",
    "sys_call", dcm_syscall,   -1,   "<system call>",
    "uni_sys",  dcm_unisys,    -1,   "[mm] [cv]",
    "?",        dcm_help,      -1,   "[command]",
    0, 0, 0, 0
};


char *long_help[] =
{
  "\n'cc' compiles a standard fixed C application program\n  -f: run in a loop [f]orever (until [k]illed).  There are no arguments for the 'cc' command.\n",
  "\n'combine' will exercise two, three, or four I/O subsystems in isolation\nof the other I/O subsystems.  For example, 'combine disk_io scsi_io' exercises\nI/O interaction specifically between the disk and the 1/4-inch tape drive\n  -f: run in a loop [f]orever (until [k]illed)\n  -r: run [r]emotely on other nodes\n ????_io: one of 'disk_io', 'scsi_io', 'tape_io', 'term_io', 'ethr_io'\n",
  "\n'do' will execute a stress-test script; that is, a file with stress test commands (*not* /bin/csh or /bin/sh commands -- see 'exec' for that). The sole\nargument to 'do' is a valid path/filename\n",
  "\n'exec' executes any valid shell level command\n",
  "\n'exit' exits the stress test and returns to the shell level\n",
  "\n'fork' will generate a process to exercise a specific I/O subsystem\n(e.g., fork disk_io).  Several forks can be initiated one after the other for some\nof the I/O subsystems\n  -f: run in a loop [f]orever (until [k]illed)\n  -r: run [r]emotely on other nodes\n ????_io: one of 'disk_io', 'scsi_io', 'tape_io', 'term_io', 'ether_io'\n  -a: ('term_io' ONLY): run terminal I/O on [a]ll ttys (0-15)\n",
  "\n'help', with no arguments, displays a list of all available stress test commands.\nWith one argument, help displays a longer usage statement for that argument\nif it is a valid stress test command\n",
  "\n'ioctl' exercises the various ioctl system call configurations.  There are no arguments to the 'ioctl' command\n",
  "\n'io_fail' invokes various system calls with invalid parameters to force the kernel\ndrivers to catch failure modes\n",
  "\n'kill' will kill a process that was invoked with the -f forever flag.\n  ?:  prints a list of all active Stress Test pid's\n  pid:  specific process id to kill\n  all:  kill all known children\n",
  "\n'nroff' will generate a process that 'nroffs' a standard text document\n  -f: run in a loop [f]orever (until [k]illed)\n",
  "\n'quit' is a synonym for 'exit'\n",
  "\n'select' is reserved for future use in the event that run-time differences\nbetween the GP1000 and the Butterfly II are necessary.  Valid arguments are 'GP1000' and 'BFII'\n",
  "\n'sst' runs a standard stress test on all five major I/O subsystems simultaneously\n, for the user who does not wish to run a customized stress test\n",
  "\n'sys_call' will invoke every major MACH system call found in section two of the Unix System Interface\nReference Manual.  Some of the calls will be invoked more than once with different parameters\n",
  "\n'uni_sys' invokes either a Uniform System matrix multiply of image convolution\nprogram.\n  -f: run in a loop [f]orever (until [k]illed)\n  mm: matrix multiply\n  ic: image convolution\n",
  "\n'?' is a synonym for 'help'\n"
};






